import ast
from collections import defaultdict
from functools import partial
import itertools
from operator import not_
from typing import Any, Dict, Mapping, Optional, Sequence, Set, Type, Union
import warnings

from opa_tools.utils import process_dct
from .utils import ALLOWED_KINDS, RequestType


def _normalize_requests_to_flatten(requests):
    if isinstance(requests, list):
        if not all(isinstance(element, dict) for element in requests):
            raise TypeError(
                "If passed as args, requests should all be dicts."
            )
        return {
            kind: dct
            for kind, dct
            in zip(ALLOWED_KINDS, requests)
        }
    if isinstance(requests, dict):
        not_allowed_kinds = [kind for kind in requests.keys() if kind not in ALLOWED_KINDS]
        if not_allowed_kinds:
            warnings.warn(
                f"Adding prefixes not listed as allowed kind: '{not_allowed_kinds}'."
            )
        return requests
    raise TypeError(
        "Requests to flatten must be passed as a list of requests to flatten "
        "or a dict whose keys are kinds, and values, requests to flatten."
    )

def flatten(
    requests = Union[Sequence[RequestType], Mapping[str, RequestType]],
    valid_types: Optional[Sequence[Type]] = None,
    type_changed_keys: Optional[Set] = None,
):
    requests = _normalize_requests_to_flatten(requests)
    flattened = {}
    for kind, request in requests.items():
        flattened.update(
            **process_dct(
                request,
                prefix=f"{kind}_",
                valid_types=valid_types,
                type_changed_keys=type_changed_keys
            )
        )
    return flattened


def unflatten(flatrequest, kinds):
    if kinds is None:
        kinds = ALLOWED_KINDS
    return tuple({
        key.removeprefix(f"{kind}_"): value
        for key, value in flatrequest.items()
        if key.startswith(f"{kind}_")
    } for kind in kinds)


def product(generating_request):
    requests = []
    generating_request_items = list(generating_request.items())
    req_keys, req_values = zip(*generating_request_items)
    for value_comb in itertools.product(*req_values):
        requests.append(dict(zip(req_keys, value_comb)))
    return requests


def wrap(requests: Sequence[Union[RequestType, Sequence[RequestType]]]):
    req_dcts = defaultdict(set)
    type_changed_keys = defaultdict(set)
    flattener = partial(flatten, valid_types=(str, int, float), type_changed_keys=type_changed_keys)
    for request in requests:
        if isinstance(request, list):
            request = flattener(request)
        assert isinstance(request, dict)
        for key, value in request.items():
            req_dcts[key].add(value)
    print(type_changed_keys)
    req_dcts = {key: list(values) for key, values in req_dcts.items()}
    req_dcts = {
        key: (
            values_lst
            if key not in type_changed_keys
            else [
                ast.literal_eval(value) if isinstance(value, str) else "??"
                for value
                in values_lst
            ]
        )
        for key, values_lst
        in req_dcts.items()
    }
    if type_changed_keys:
        warnings.warn(
            "Please, review and correct the types for the "
            "following keys in the wrapped request:"
            "\n- " + "\n- ".join(sorted(type_changed_keys))
        )
    return req_dcts
